#include <cstdlib>
#define __STDC_LIMIT_MACROS
#define __STDC_CONSTANT_MACROS
#include <stdint.h>
#undef _GLIBCXX_ATOMIC_BUILTINS

#include <iostream>
#include <vector>

#include <clang/Basic/Diagnostic.h>
#include <clang/Frontend/DiagnosticOptions.h>
#include <clang/Frontend/CompilerInstance.h>
#include <clang/Frontend/CompilerInvocation.h>
#include <clang/Frontend/Utils.h>
#include <clang/Frontend/ASTUnit.h>

int main( int argc, char ** argv ) {
   using namespace clang ;
   using namespace llvm ;
   // Initialize compiler options list
   std::vector< const char *> args;
   for( int n = 1; n < argc; ++ n )
      args.push_back( argv[ n ] );
   args.push_back( "main_clang.cpp" );
   const char** opts = &args.front();
   int opts_num = args.size();

   // Create and setup diagnostic consumer
   DiagnosticOptions diagOpts;
   IntrusiveRefCntPtr< DiagnosticsEngine> diags( CompilerInstance::createDiagnostics(
         diagOpts, // Opts
         opts_num, // Argc
         opts,     // Argv
         0,        // Client
         true,     // ShouldOwnClient
         false     // ShouldCloneClient
   ) );

   // Create compiler invocation
   IntrusiveRefCntPtr<CompilerInvocation> compInvoke = clang::createInvocationFromCommandLine(
         makeArrayRef( opts, opts + opts_num ), // Args
         diags                                  // Diags
   );
   if( !compInvoke ) {
      std::cout << "Can't create compiler invocation for given args" ;
      return -1;
   }
   // Parse file
   clang::ASTUnit *tu = ASTUnit ::LoadFromCompilerInvocation(
         compInvoke.getPtr(), // CI
         diags,               // Diags
         false,               // OnlyLocalDecls
         true,                // CaptureDiagnostics
         false,               // PrecompilePreamble
         TU_Complete,         // TUKind
         false                // CacheCodeCompletionResults
   );
   if( tu == 0 )
      std::cout << "Translation unit was not created" ;
   else
      std::cout << "Translation unit successfully created" ;
   return 0; 
}
